#!/usr/bin/perl
# TitleCase.pl
# Version: 3.0a
# Last Updated: 26 November 2017

use strict;
use warnings;
use utf8;
use open qw( :encoding(UTF-8) :std );

my @data = (
"Hashtag Tested as #MeToo #mrp",
"What about \@mrp \@nOhSay?",
"this too \"and don't\"",
"C.I.A.A.P. the CIA and C.I.S.",
"wonder\@mrpasini.com",
"2lmc Spool: 'Gruber on OmniFocus and Vapo(u)rware'",
"Have you read “The Lottery”?",
"People probably won't put http://foo.com/bar/ in titles",
"Scott Moritz and TheStreet.com’s million iPhone la‑la land",
"BlackBerry vs. iPhone",
"Notes and observations regarding Apple from ‘The Beat Goes On’",
"Apple from 'The Beat Goes On'",
"What is AT&T's problem?",
"Apple deal with AT&T falls through",
"The SEC's Apple probe: what you need to know",
"Never touch paths like /var/run before/after /boot",
);

my $mask = "Before: %s\nAfter:  %s\n\n";

for my $datum (@data) { 
    printf $mask, $datum, tc($datum);
}

sub tc {

	local $_ = shift;

		# NYT list of small words not to be title capped
	my @small_words = qw( (?<!q&)a an and as at(?!&t) but by en for if in of on or the to v[.]? via vs[.]? );
	my $small_re = join '|', @small_words;

		# fix uppercase after apostrophe
	my $apos = qr/ (?: ['’] [[:lower:]]* )? /x;

	s{\A\s+}{}, s{\s+\z}{};				# remove whitespace at beg/end of string
	$_ = lc $_ if not /[[:lower:]]/;	# lowercase string if all caps

		# main protections, substitution
	s{
		\b (_*) (?:
			( (?<=[ ][/\\@#]) [[:alpha:]]+ [-_[:alpha:]/\\]+ | #2 path, @, hashtag
				[-_[:alpha:]]+ [@.:] [-_[:alpha:]@.:/]+ $apos ) #2 URL/domain/email
			|
			( (?i: $small_re ) $apos )                         #3 small word
			|
			( [[:alpha:]] [[:lower:]'’()\[\]{}]* $apos )       #4 no internal caps
			|
			( [[:alpha:]] [[:alpha:]'’()\[\]{}]* $apos )       #5 some other word
		) (_*) \b
		}{
		$1 . (
		  defined $2 ? $2       # preserve URL, domain, or email
		: defined $3 ? "\L$3"   # lowercase small word
		: defined $4 ? "\u\L$4" # capitalize word w/o internal caps
		: $5	      				# preserve other kinds of word
		) . $6
	}xeg;
		# Exceptions for small words: capitalize at start and end of title
	s{
		(  \A [[:punct:]]*         # start of title...
		|  [:.;?!][ ]+             # or of subsentence...
		|  [ ]['"“‘(\[][ ]*     )  # or of inserted subphrase...
		( $small_re ) \b           # ... followed by small word
		}{$1\u\L$2}xig;
		s{
		\b ( $small_re )      # small word...
		(?= [[:punct:]]* \Z   # ... at the end of the title...
		|   ['"’”)\]] [ ] )   # ... or of an inserted subphrase?
	}{\u\L$1}xig;
		#Exceptions for acronyms with periods
	s{
		(?<=\b) ( ([[:alpha:]]) \. \b)
	}{\U$2}xig;
	s{
		([[:upper:]]{2,}) \.
	}{$1}xig;
		# Exceptions for small words in hyphenated compound words
		## e.g. "in-flight" -> In-Flight
	s{
	\b
	(?<! -)		# Negative lookbehind for a hyphen;
					# Don't match man-in-the-middle but do  match in-flight
	( $small_re )
	(?= -[[:alpha:]]+)		# lookahead for "-someword"
	}{\u\L$1}xig;
		## # e.g. "Stand-in" -> "Stand-In" (Stand is already capped at this point)
	s{
	\b
	(?<!…)					# Negative lookbehind for a hyphen; we don't want to match man-in-the-middle but do want (stand-in)
	( [[:alpha:]]+- )		# $1 = first word and hyphen, should already be properly capped
	( $small_re )           # ... followed by small word
	(?!	- )					# Negative lookahead for another '-'
	}{$1\u$2}xig;

	return $_;
}
